package es.prodevelop.android.common.preferences;

import android.content.Context;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;

/**
 * Utility class to store values in a shared preferences file.
 * 
 * @author aromeu
 * 
 */
public class PreferenceWriter {

	private Context mContext;
	private String mSharedPreferencesName;

	private SharedPreferences mPreferences;

	public PreferenceWriter(Context aContext, String aSharedPreferencesName) {
		this.mContext = aContext;
		this.mSharedPreferencesName = aSharedPreferencesName;

		this.mPreferences = mContext.getSharedPreferences(
				mSharedPreferencesName, Context.MODE_WORLD_WRITEABLE);
	}

	private Editor getEditor() {
		return this.mPreferences.edit();
	}

	public void clear() {
		final Editor e = getEditor();
		e.clear();
		e.commit();
	}

	public void putBoolean(String key, boolean value) {
		final Editor e = getEditor();
		e.putBoolean(key, value);
		e.commit();
	}

	public void putFloat(String key, float value) {
		final Editor e = getEditor();
		e.putFloat(key, value);
		e.commit();
	}

	public void putInt(String key, int value) {
		final Editor e = getEditor();
		e.putInt(key, value);
		e.commit();
	}

	public void putLong(String key, long value) {
		final Editor e = getEditor();
		e.putLong(key, value);
		e.commit();
	}

	public void putString(String key, String value) {
		final Editor e = getEditor();
		e.putString(key, value);
		e.commit();
	}

	public void remove(String key) {
		final Editor e = getEditor();
		e.remove(key);
		e.commit();
	}
}
